local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")

local gui, lootRollGUI, lootGUI
local old, checkboxes = {}, {}
local Version = GetAddOnMetadata("NDui_Plus", "Version")

local function copyTable(source, target)
	for key, value in pairs(source) do
		if type(value) == "table" then
			target[key] = {}
			for k in pairs(value) do
				target[key][k] = value[k]
			end
		else
			target[key] = value
		end
	end
end

local function addSubCategory(parent, name)
	local header = parent:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	header:SetText(name)

	local line = parent:CreateTexture(nil, "ARTWORK")
	line:SetSize(320, 1)
	line:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 0, -4)
	line:SetColorTexture(1, 1, 1, .2)

	return header
end

local modConver = {
	[1] = 0,
	[0] = 1,
}

local function toggle(f)
	NDuiPlusDB[f.value] = f:GetChecked()
end

local function createToggleBox(parent, option, category, index)
	local value, text, tooltip, data = unpack(option)
	local f = B.CreateCheckBox(parent)
	f:SetHitRectInsets(-5, -55, -5, -5)
	f.name = B.CreateFS(f, 14, text, false, "LEFT", 30, 0)
	f.value = value
	f:SetScript("OnClick", toggle)

	if tooltip then
		B.AddTooltip(f, "ANCHOR_RIGHT", tooltip, "info")
	end
	if data and type(data) == "function" then
		local bu = B.CreateGear(parent)
		bu:SetPoint("LEFT", f.name, "RIGHT", -2, 1)
		bu:SetScript("OnClick", data)
		if not NDuiPlusDB[value] then bu:Hide() end
	end

	local xoffset = modConver[mod(index, 2)]*180
	local yoffset = -(floor(index/2.1)*32 + 20)
	f:SetPoint("TOPLEFT", category, "BOTTOMLEFT", xoffset, yoffset)

	tinsert(checkboxes, f)
	return f
end

local extraGUIs = {}
local function createExtraGUI(parent, name, title, bgFrame)
	local frame = CreateFrame("Frame", name, parent)
	frame:SetSize(260, parent:GetHeight())
	frame:SetPoint("TOPLEFT", parent, "TOPRIGHT", 3, 0)
	B.SetBD(frame)
	parent:HookScript("OnHide", function()
		if frame:IsShown() then frame:Hide() end
	end)

	if title then
		B.CreateFS(frame, 14, title, "system", "TOPLEFT", 20, -25)
	end

	if bgFrame then
		frame.bg = CreateFrame("Frame", nil, frame)
		frame.bg:SetSize(frame:GetWidth() - 20, frame:GetHeight() - 60)
		frame.bg:SetPoint("TOPLEFT", 10, -50)
		B.CreateBD(frame.bg, .3)
	end

	tinsert(extraGUIs, frame)
	return frame
end

local function toggleExtraGUI(name)
	for _, frame in next, extraGUIs do
		if frame:GetName() == name then
			ToggleFrame(frame)
		else
			frame:Hide()
		end
	end
end

local function createOptionTitle(parent, title, offset)
	B.CreateFS(parent, 14, title, nil, "TOP", 0, offset)
	if B.SetGradient then
		local line = B.SetGradient(parent, "H", 1, 1, 1, .25, .25, 200, C.mult)
		line:SetPoint("TOPLEFT", 30, offset-20)
	else
		local l = CreateFrame("Frame", nil, parent)
		l:SetPoint("TOPLEFT", 30, offset-20)
		B.CreateGF(l, 200, C.mult, "Horizontal", 1, 1, 1, .25, .25)
	end
end

local function sliderValueChanged(self, v)
	local current = tonumber(format("%.0f", v))
	self.value:SetText(current)
	NDuiPlusDB[self.__value] = current
	self.__update()
end

local function createOptionSlider(parent, title, minV, maxV, defaultV, x, y, value, func)
	local slider = B.CreateSlider(parent, title, minV, maxV, 1, x, y)
	slider:SetValue(NDuiPlusDB[value])
	slider.value:SetText(NDuiPlusDB[value])
	slider.__value = value
	slider.__update = func
	slider.__default = defaultV
	slider:SetScript("OnValueChanged", sliderValueChanged)
end

local function createOptionCheck(parent, offset, text)
	local box = B.CreateCheckBox(parent)
	box:SetPoint("TOPLEFT", 10, -offset)
	B.CreateFS(box, 14, text, false, "LEFT", 30, 0)
	return box
end

local function createDropDown(parent, width, height, value, data)
	local dd = B.CreateDropDown(parent, width, height, data)
	dd.Text:SetText(data[NDuiPlusDB[value]])
	local opt = dd.options
	dd.button:HookScript("OnClick", function()
		for num = 1, #data do
			if num == NDuiPlusDB[value] then
				opt[num]:SetBackdropColor(1, .8, 0, .3)
				opt[num].selected = true
			else
				opt[num]:SetBackdropColor(0, 0, 0, .3)
				opt[num].selected = false
			end
		end
	end)

	for i in pairs(data) do
		opt[i]:HookScript("OnClick", function()
			NDuiPlusDB[value] = i
		end)
	end
	B.CreateFS(dd, 14, name, "system", "CENTER", 0, 25)

	return dd
end

local function setupLoot()
	toggleExtraGUI("NDuiPlusGUI_Loot")
	if lootGUI then return end

	lootGUI = createExtraGUI(NDuiPlusGUI, "NDuiPlusGUI_Loot", L["LootEnhanced Settings"], true)

	local frame = lootGUI.bg

	createOptionTitle(frame, L["Loot Announce"], -10)

	local lootOptions = {
		[1] = {"LootAnnounce", L["Enable Loot Announce"]},
		[2] = {"LootAnnounceTitle", L["Announce Target Name"]},
	}

	local offset = 60
	for _, option in ipairs(lootOptions) do
		local value, text = unpack(option)
		local box = createOptionCheck(frame, offset, text)
		box:SetChecked(NDuiPlusDB[value])
		box.value = value
		box:SetScript("OnClick", toggle)

		offset = offset + 35
	end

	local qualitys = {
		[1] = ITEM_QUALITY_COLORS[1].hex .. ITEM_QUALITY1_DESC .. "|r",
		[2] = ITEM_QUALITY_COLORS[2].hex .. ITEM_QUALITY2_DESC .. "|r",
		[3] = ITEM_QUALITY_COLORS[3].hex .. ITEM_QUALITY3_DESC .. "|r",
		[4] = ITEM_QUALITY_COLORS[4].hex .. ITEM_QUALITY4_DESC .. "|r",
	}

	local dd = createDropDown(frame, 180, 28, "LootAnnounceRarity", qualitys)
	B.CreateFS(dd, 14, L["Rarity Threshold"], false, "CENTER", 0, 25)
	dd:SetPoint("TOPLEFT", 30, -160)
end

local function setupLootRoll()
	toggleExtraGUI("NDuiPlusGUI_LootRoll")
	if lootRollGUI then return end

	lootRollGUI = createExtraGUI(NDuiPlusGUI, "NDuiPlusGUI_LootRoll", L["LootRoll Settings"], true)

	local frame = lootRollGUI.bg
	local defaultValue = {328, 28}

	local function updateSize()
		if not _G.NDuiPlus_LootRoll then
			P:LootRollTest()
		end
		local testFrame = _G.NDuiPlus_LootRoll
		if not testFrame then return end
		local width, height = NDuiPlusDB["LootRollWidth"], NDuiPlusDB["LootRollHeight"]
		local fontSize = height / 2
		testFrame:Show()
		testFrame:SetSize(width, height)
		testFrame.button:SetSize(height - (C.mult*2), height - (C.mult*2))
		testFrame.fsbind:SetFont(DB.Font[1], fontSize, DB.Font[3])
		testFrame.fsloot:SetFont(DB.Font[1], fontSize, DB.Font[3])
		testFrame.needbutt:SetSize(height-4, height-4)
		testFrame.greedbutt:SetSize(height-4, height-4)
		testFrame.passbutt:SetSize(height-4, height-4)
	end

	createOptionTitle(frame, L["Frame Size"], -10)
	createOptionSlider(frame, L["Frame Width"], 200, 500, defaultValue[1], 20, -70, "LootRollWidth", updateSize)
	createOptionSlider(frame, L["Frame Height"], 20, 50, defaultValue[2], 20, -140, "LootRollHeight", updateSize)
	createOptionTitle(frame, L["Growth Direction"], -200)
	local dd = createDropDown(frame, 180, 28, "LootRollDirection", {L["Up"], L["Down"]})
	dd:SetPoint("TOPLEFT", 30, -240)

	lootRollGUI:HookScript("OnHide", function()
		if _G.NDuiPlus_LootRoll then _G.NDuiPlus_LootRoll:Hide() end
	end)
end

local featuresList = {
	[1] = {"normTex", L["ReplaceTexture"], L["ReplaceTextureTip"]},
	[2] = {"Loot", L["LootEnhanced"], L["LootEnhancedTip"], setupLoot},
	[3] = {"ChatEmote", L["ChatEmote"]},
	[4] = {"LootRoll", L["LootRoll"], L["LootRollTip"], setupLootRoll},
	[5] = {"OfflineBag", L["OfflineBag"], L["OfflineBagTip"]},
	[6] = {"ExtendedGuildUI", L["ExtendedGuildUI"], L["ExtendedGuildUITip"]},
}

local skinsList = {
	[1] = {"Ace3", "AceGUI-3.0"},
	[2] = {"InboxMailBag", "Inbox MailBag"},
	[3] = {"MerInspect", "MerInspect"},
	[4] = {"alaGearMan", "alaGearMan"},
	[5] = {"ClassicThreatMeter", "ThreatClassic2"},
	[6] = {"Spy", "Spy"},
	[7] = {"ButtonForge", "Button Forge"},
	[8] = {"MeetingHorn", "MeetingHorn"},
	[9] = {"GearMenu", "GearMenu"},
	[10] = {"alaCalendar", "alaCalendar"},
	[11] = {"ls_Toasts", "ls_Toasts"},
	[12] = {"WIM", "WIM"},
	[13] = {"ItemRack", "ItemRack"},
	[14] = {"WhisperPop", "WhisperPop"},
	[15] = {"AutoBarClassic", "AutoBar"},
	[16] = {"AtlasLootClassic", "AtlasLoot"},
	[17] = {"Immersion", "Immersion"},
	[18] = {"Skillet", "Skillet"},
}

local guiRefresh = function()
	for i = 1, #checkboxes do
		checkboxes[i]:SetChecked(NDuiPlusDB[checkboxes[i].value] == true)
	end
end

local function CreateGUI()
	if gui then gui:Show() return end

	gui = CreateFrame("Frame", "NDuiPlusGUI", UIParent)
	tinsert(UISpecialFrames, "NDuiPlusGUI")
	gui:SetWidth(370)
	gui:SetHeight(480)
	gui:SetPoint("CENTER")
	gui:SetFrameStrata("HIGH")
	gui:SetFrameLevel(5)
	B.CreateMF(gui)
	B.SetBD(gui)
	B.CreateFS(gui, 18, "NDui_Plus", true, "TOP", 0, -10)
	B.CreateFS(gui, 16, "v"..Version, false, "TOP", 0, -30)

	local scroll = CreateFrame("ScrollFrame", nil, gui, "UIPanelScrollFrameTemplate")
	scroll:SetPoint("TOPLEFT", 0, -60)
	scroll:SetPoint("BOTTOMRIGHT", -30, 50)
	scroll.child = CreateFrame("Frame", nil, scroll)
	scroll.child:SetSize(340, 1)
	scroll:SetScrollChild(scroll.child)
	B.ReskinScroll(scroll.ScrollBar)

	local offset = ceil(#skinsList/2)*32 + 175
	local blank = CreateFrame("Frame", nil, scroll.child)
	blank:SetSize(10, 10)
	blank:SetPoint("TOP", 0, -offset)

	local features = addSubCategory(scroll.child, L["Features"])
	features:SetPoint("TOPLEFT", 20, -10)

	local skins = addSubCategory(scroll.child, L["Skins"])
	skins:SetPoint("TOPLEFT", features, "BOTTOMLEFT", 0, -120)

	for index, option in ipairs(featuresList) do
		createToggleBox(scroll.child, option, features, index)
	end

	for index, option in ipairs(skinsList) do
		createToggleBox(scroll.child, option, skins, index)
	end

	local close = B.CreateButton(gui, 80, 20, CLOSE)
	close:SetPoint("BOTTOMRIGHT", -20, 15)
	close:SetScript("OnClick", function()
		copyTable(old, NDuiPlusDB)
		guiRefresh()
		gui:Hide()
	end)

	local okay = B.CreateButton(gui, 80, 20, OKAY)
	okay:SetPoint("RIGHT", close, "LEFT", -10, 0)
	okay:SetScript("OnClick", function()
		copyTable(NDuiPlusDB, old)
		StaticPopup_Show("RELOAD_NDUI")
		gui:Hide()
	end)

	copyTable(NDuiPlusDB, old)
	guiRefresh()
end

SlashCmdList['NDUI_PLUS'] = function(msg)
	local status = P:VersionCheck_Compare(DB.Version, P.SupportVersion)
	if status == "IsOld" then
		P:Print(format(L["Version Check"], P.SupportVersion))
		return
	end

	if msg:lower() == "debug" then
		NDuiPlusDB["debug"] = not NDuiPlusDB["debug"]
		_G.DEFAULT_CHAT_FRAME:AddMessage("|cFF70B8FFNDui_Plus:|r Debug " .. format(NDuiPlusDB["debug"] and "on" or "off"))
	else
		CreateGUI()
	end
end
SLASH_NDUI_PLUS1 = "/ndp"
SLASH_NDUI_PLUS2 = "/nduiplus"