local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")

local _G = getfenv(0)
local select, pairs, type = select, pairs, type
----------------------------
-- Credit: AddOnSkins_MeetingStone by hokohuang
----------------------------
function P:MeetingHorn()
	if not IsAddOnLoaded("MeetingHorn") then return end
	if not NDuiPlusDB["MeetingHorn"] then return end

	local mainFrame = _G.MeetingHornMainPanel
	if not mainFrame then return end
	B.ReskinPortraitFrame(mainFrame)
	MeetingHornMainPanelPortraitFrame:SetAlpha(0)

	for _, tab in ipairs(mainFrame.Tabs) do
		B.ReskinTab(tab)
		local text = tab.Text or _G[tab:GetName().."Text"]
		if text then
			text:SetPoint("CENTER", tab)
		end
	end

	local function strToPath(str)
		local path = {}
		for v in string.gmatch(str, "([^\.]+)") do 
			table.insert(path, v)
		end
		return path
	end

	local function getValue(pathStr, tbl)
		local keys = strToPath(pathStr) 
		local value
		for _, key in pairs(keys) do
			value = value and value[key] or tbl[key]
		end
		return value
	end

	local Dropdowns = {
		"Browser.Activity",
		"Browser.Mode",
		"Browser.Quick",
		"Manage.Creator.Activity",
		"Manage.Creator.Mode",
	}

	local Headers = {
		"Browser",
		"Manage.Applicant",
	}

	local ScrollBars = {
		"Browser.ActivityList.scrollBar",
		"Manage.Applicant.ApplicantList.scrollBar",
		"Options.Filters.FilterList.scrollBar",
		"FeedBack.EditBox.ScrollFrame.ScrollBar",
		"Manage.Creator.Comment.ScrollFrame.ScrollBar",
		"Manage.Chat.ChatFrame.scrollBar",
	}

	local Panels = {
		"Browser",
		"Manage.Creator",
		"Manage.Chat.ChatBg",
		"Manage.Creator.Comment",
		"Manage.Applicant",
		"Help",
		"Options.Options",
		"Options.Filters",
	}

	local Buttons = {
		"Browser.Reset",
		"Browser.Refresh",
		"Manage.Creator.CreateButton",
		"Manage.Creator.CloseButton",
		"Options.Filters.Add",
		"Options.Filters.Import",
		"Options.Filters.Export",
		"FeedBack.AcceptButton",
		"FeedBack.CancelButton",
	}

	for _, v in pairs(Dropdowns) do
		local dropdown = getValue(v, mainFrame)
		if dropdown then
			B.StripTextures(dropdown)
			local down = dropdown.MenuButton
			down:ClearAllPoints()
			down:SetPoint("RIGHT", -18, 0)
			B.ReskinArrow(down, "down")
			down:SetSize(20, 20)

			local bg = B.CreateBDFrame(dropdown, 0)
			bg:SetPoint("TOPLEFT", 0, -2)
			bg:SetPoint("BOTTOMRIGHT", -18, 2)
			B.CreateGradient(bg)
		end
	end

	for _, v in pairs(Headers) do
		local headerParent = getValue(v, mainFrame)
		if headerParent then
			local index = 1
			local header = headerParent["Header"..index]
			while header do
				header:DisableDrawLayer("BACKGROUND")
				local bg = B.CreateBDFrame(header, .25)
				bg:SetPoint("BOTTOMRIGHT", -2, C.mult)
				header:SetHighlightTexture(DB.bdTex)
				local hl = header:GetHighlightTexture()
				hl:SetAllPoints(bg)
				hl:SetVertexColor(DB.r, DB.g, DB.b, .25)
				index = index + 1
				header = headerParent["Header"..index]
			end
		end
	end

	for _, v in pairs(ScrollBars) do
		local scrollBar = getValue(v, mainFrame)
		if scrollBar then
			B.ReskinScroll(scrollBar)
		end
	end

	for _, v in pairs(Panels) do
		local panel = getValue(v, mainFrame)
		if panel then
			panel:DisableDrawLayer("BACKGROUND")
			panel:DisableDrawLayer("BORDER")
			local bg = B.CreateBDFrame(panel, .25)
			bg:SetPoint("TOPLEFT", 0, 0)
			bg:SetPoint("BOTTOMRIGHT", 0, 0)
		end
	end

	for _, v in pairs(Buttons) do
		local button = getValue(v, mainFrame)
		if button then
			B.Reskin(button)
			if button.LeftSeparator then button.LeftSeparator:SetAlpha(0) end
			if button.RightSeparator then button.RightSeparator:SetAlpha(0) end
		end
	end

	local input = getValue("Browser.Input", mainFrame)
	if input then
		B.ReskinInput(input)
	end

	local MeetingHorn = LibStub('AceAddon-3.0'):GetAddon('MeetingHorn')
	local ListView = MeetingHorn:GetClass('UI.ListView')

	hooksecurefunc(ListView, "GetButton", function(self, index)
		local button = self._buttons[index]
		if button and button.Signup and not button.styled then
			button:SetHighlightTexture(DB.bdTex)
			local hl = button:GetHighlightTexture()
			hl:SetVertexColor(DB.r, DB.g, DB.b, .25)
			B.Reskin(button.Signup)
			button.styled = true
		end
	end)

	if IsAddOnLoaded("tdInspect") then  -- Credit: tdUI
		local tdInspect = LibStub('AceAddon-3.0'):GetAddon('tdInspect')
		local Browser = MeetingHorn:GetClass('UI.Browser')
		local Inspect = tdInspect:GetModule('Inspect')

		local hook = function(orig, self, activity)
			local r = orig(self, activity)
			tinsert(r, 3, {
				text = INSPECT,
				func = function()
					Inspect:Query(nil, activity:GetLeader())
				end,
			})
			return r
		end

		local origCreateActivityMenu = Browser.CreateActivityMenu

		Browser.CreateActivityMenu = function(...)
			return hook(origCreateActivityMenu, ...)
		end
	end

	local progressBar = mainFrame.Browser.ProgressBar
	if progressBar then
		B.StripTextures(progressBar)
		progressBar:SetStatusBarTexture(DB.normTex)
		progressBar:DisableDrawLayer("BACKGROUND")
		B.CreateBDFrame(progressBar, .25)
	end

	local DataBroker = _G.MeetingHornDataBroker
	if DataBroker then
		DataBroker:DisableDrawLayer("BACKGROUND")
		DataBroker:SetSize(158, 32)
		B.SetBD(DataBroker, C.mult, -C.mult, -C.mult, C.mult)
		DataBroker.Text:SetPoint("CENTER", 16, 0)
		local logo = DataBroker:CreateTexture(nil, "ARTWORK")
		logo:SetTexture("Interface\\AddOns\\MeetingHorn\\Media\\Logo2")
		logo:SetSize(30, 30)
		logo:SetPoint("LEFT", 12, 0)
	end
end
