local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")

local _G = getfenv(0)
local select, pairs, type = select, pairs, type

local function reskinButtons(frame)
	for i = 1, frame:GetNumChildren() do
		local child = select(i, frame:GetChildren())
		if child:GetObjectType() == 'Button' and child:GetText() then
			B.Reskin(child)
		end
	end
end

function P:WeakAuras()
	if not IsAddOnLoaded("WeakAuras") then return end

	reskinButtons(WeakAurasTooltipAnchor)
	B.Reskin(WeakAurasTooltipImportButton)

	local function ReskinWAOptions()
		local frame = _G.WeakAurasOptions
		if not frame or frame.styled then return end

		B.StripTextures(frame)
		B.SetBD(frame)
		B.ReskinInput(frame.filterInput)
		B.Reskin(_G.WASettingsButton)

		-- Minimize, Close
		for i = 1, frame:GetNumChildren() do
			local child = select(i, frame:GetChildren())
			local numRegions = child:GetNumRegions()
			local numChildren = child:GetNumChildren()
			
			if numRegions == 3 and numChildren == 1 and child.PixelSnapDisabled then
				B.StripTextures(child)
				
				local button = child:GetChildren()
				local texturePath = button.GetNormalTexture and button:GetNormalTexture():GetTexture()
				if texturePath and texturePath == "Interface\\BUTTONS\\UI-Panel-CollapseButton-Up" then
					B.Reskin(button)
					button.SetNormalTexture = B.Dummy
					button.SetPushedTexture = B.Dummy
					button:SetSize(18, 18)
					button:ClearAllPoints()
					button:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -30, -6)

					local tex = button:CreateTexture(nil, "ARTWORK")
					tex:SetAllPoints()
					B.SetupArrow(tex, "up")
					button.bgTex = tex

					button:SetScript("OnEnter", B.Texture_OnEnter)
					button:SetScript("OnLeave", B.Texture_OnLeave)
					button:HookScript("OnClick",function(self)
						if frame.minimized then
							B.SetupArrow(self.bgTex, "down")
						else
							B.SetupArrow(self.bgTex, "up")
						end
					end)
				else
					B.ReskinClose(button, "TOPRIGHT", frame, "TOPRIGHT", -6, -6)
					button:SetSize(18, 18)
				end
			end
		end

		-- childGroups
		local childGroups = {
			"texturePicker",
			"iconPicker",
			"modelPicker",
			"importexport",
			"texteditor",
			"codereview",
		}

		for _, key in pairs(childGroups) do
			local group = frame[key]
			if group then
				reskinButtons(group.frame)
			end
		end

		-- WeakAurasSnippets
		local snippets = _G.WeakAurasSnippets
		B.StripTextures(snippets)
		B.SetBD(snippets)
		reskinButtons(snippets)

		frame.styled = true
	end

	local function loadFunc(event, addon)
		if addon == "WeakAurasOptions" then
			hooksecurefunc(WeakAuras, "ShowOptions", ReskinWAOptions)
			B:UnregisterEvent(event, loadFunc)
		end
	end
	B:RegisterEvent("ADDON_LOADED", loadFunc)
end
